﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections;
using Telerik.OpenAccess;
using Telerik.Web.UI;
using OpenAccessData;

namespace CarRentWebSite
{
    public partial class Admin : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (Session["newOrUpdated"] == null)
            {
                Session["newOrUpdated"] = new List<int>();
            }
            RadGrid1.ItemCreated += new GridItemEventHandler(RadGrid1_ItemCreated);
            if (Session["container"] == null)
            {
                ObjectContainer container = new ObjectContainer();
                CarRentalAdministratorService service = new CarRentalAdministratorService();
                container.Apply(service.GetAllCars());
                Session["container"] = container;
            }
            else
            {
                ObjectContainer container = (ObjectContainer)Session["container"];
            }
        }

        void RadGrid1_ItemCreated(object sender, GridItemEventArgs e)
        {
            if (!(e.Item.DataItem is GridInsertionObject))
            {
                Car c = (Car)(e.Item.DataItem);
                if (c != null)
                {
                    if (((List<int>)Session["newOrUpdated"]).Contains(c.CarID))
                    {
                        e.Item.BackColor = System.Drawing.Color.Coral;
                    }
                }
            }
        }

        protected void RadGrid1_NeedDataSource(object source, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
        {
            ObjectContainer container = (ObjectContainer)Session["container"];
            RadGrid1.DataSource = LoadCars(container);
            if (Session["dirtyCount"] != null)
            {
                Label2.Text = Session["dirtyCount"].ToString(); 
            }

        }

        private IList LoadCars(Telerik.OpenAccess.ObjectContainer container)
        {
            return container.NamedList("Cars");
        }

        protected void Button1_Click(object sender, EventArgs e)
        {
            ObjectContainer container = (Telerik.OpenAccess.ObjectContainer)Session["container"];
            ObjectContainer.ChangeSet changedData = container.GetChanges(Telerik.OpenAccess.ObjectContainer.Verify.Changed);
            CarRentalAdministratorService service = new CarRentalAdministratorService();
            container.Apply(service.SyncCars(changedData));
            Session["dirtyCount"] = 0;
            Label2.Text = "0";
            ((List<int>)Session["newOrUpdated"]).Clear();
            RadGrid1.Rebind();

        }

        protected void RadGrid1_DeleteCommand(object source, Telerik.Web.UI.GridCommandEventArgs e)
        {
            string carID = ((GridDataItem)e.Item).GetDataKeyValue("CarID").ToString();
            ObjectContainer container = (ObjectContainer)Session["container"];
            container.Transaction.Begin();
            Car car = (Car)container.GetObjectById(Database.OID.ParseObjectId(typeof(Car), carID));
            container.Remove(car);
            container.NamedList("Cars").Remove(car);
            container.Transaction.Commit();
            IncCounter();
        }

        protected void RadGrid1_UpdateCommand(object source, GridCommandEventArgs e)
        {
            string carID = ((GridEditFormItem)e.Item).ParentItem.GetDataKeyValue("CarID").ToString();
            ObjectContainer container = (ObjectContainer)Session["container"];
            container.Transaction.Begin();
            Car car = (Car)container.GetObjectById(Database.OID.ParseObjectId(typeof(Car), carID));
            if (car != null)
            {
                if (!container.IsDirty(car))
                {
                    IncCounter();
                }
            }
            ((GridEditableItem)e.Item).UpdateValues(car);
            container.Transaction.Commit();
            ((List<int>)Session["newOrUpdated"]).Add(int.Parse(carID));

        }

        protected void RadGrid1_InsertCommand(object source, GridCommandEventArgs e)
        {
            ObjectContainer container = (ObjectContainer)Session["container"];
            container.Transaction.Begin();
            Car newCar = new Car();
            ((GridEditableItem)e.Item).UpdateValues(newCar);
            container.Add(newCar);
            container.NamedList("Cars").Add(newCar);
            container.Transaction.Commit();
            IncCounter();
            container.Transaction.Begin();
            ((List<int>)Session["newOrUpdated"]).Add(newCar.CarID);
            container.Transaction.Commit();
          
        }


        private void IncCounter()
        {
            int dirtyCount;
            if (Session["dirtyCount"] != null)
            {
                dirtyCount = (int)Session["dirtyCount"];
                dirtyCount++;
                Session["dirtyCount"] = dirtyCount;
            }
            else
            {
                dirtyCount = 1;
                Session["dirtyCount"] = dirtyCount;
            }

        }


    }
}
